Package: BiocSet
Title: Representing Different Biological Sets
Version: 1.24.0
Authors@R: c(
    person("Kayla", "Morrell", email = "kayla.morrell16@gmail.com",
        role = c("aut", "cre")),
    person("Martin", "Morgan", role = "aut"),
    person("Kevin", "Rue-Albrecht", role = "ctb"),
    person("Lluís", "Revilla Sancho", role = "ctb"))
Description: BiocSet displays different biological sets in a triple tibble
    format. These three tibbles are `element`, `set`, and `elementset`. The
    user has the abilty to activate one of these three tibbles to perform common
    functions from the dplyr package. Mapping functionality and accessing web
    references for elements/sets are also available in BiocSet.
Depends: R (>= 3.6), dplyr
Imports: methods, tibble, utils, rlang, plyr, S4Vectors, BiocIO,
        AnnotationDbi, KEGGREST, ontologyIndex, tidyr
Suggests: GSEABase, airway, org.Hs.eg.db, DESeq2, limma, BiocFileCache,
        GO.db, testthat, knitr, rmarkdown, BiocStyle
biocViews: GeneExpression, GO, KEGG, Software
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BiocSet
git_branch: RELEASE_3_22
git_last_commit: f0c8b16
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:49:21 UTC; biocbuild
Author: Kayla Morrell [aut, cre],
  Martin Morgan [aut],
  Kevin Rue-Albrecht [ctb],
  Lluís Revilla Sancho [ctb]
Maintainer: Kayla Morrell <kayla.morrell16@gmail.com>
