# Generated by roxygen2: do not edit by hand

S3method(arrange,BiocSet)
S3method(as.list,BiocSet)
S3method(es_activate,BiocSet)
S3method(filter,BiocSet)
S3method(group_by,BiocSet)
S3method(intersect,BiocSet)
S3method(left_join,BiocSet)
S3method(map_set,BiocSet)
S3method(mutate,BiocSet)
S3method(select,BiocSet)
S3method(summarise,BiocSet)
S3method(union,BiocSet)
export("metadata<-")
export(BiocSet)
export(BiocSet_from_GeneSetCollection)
export(BiocSet_from_elementset)
export(GeneSetCollection_from_BiocSet)
export(OBOSet)
export(arrange_element)
export(arrange_elementset)
export(arrange_set)
export(data.frame_from_element)
export(data.frame_from_elementset)
export(data.frame_from_set)
export(es_activate)
export(filter_element)
export(filter_elementset)
export(filter_set)
export(go_sets)
export(intersect_single)
export(kegg_sets)
export(left_join_element)
export(left_join_elementset)
export(left_join_set)
export(map_add_element)
export(map_add_set)
export(map_multiple)
export(map_set)
export(map_unique)
export(metadata)
export(mutate_element)
export(mutate_elementset)
export(mutate_set)
export(oboset_element_ancestors)
export(oboset_element_children)
export(oboset_element_parents)
export(oboset_set_ancestors)
export(oboset_set_children)
export(oboset_set_parents)
export(select_element)
export(select_elementset)
export(select_set)
export(summarise_element)
export(summarise_elementset)
export(summarise_set)
export(tibble_from_element)
export(tibble_from_elementset)
export(tibble_from_set)
export(union_single)
export(url_ref)
export(url_ref_element)
export(url_ref_set)
exportClasses(BiocSet)
exportClasses(OBOSet)
exportMethods(coerce)
exportMethods(es_element)
exportMethods(es_elementset)
exportMethods(es_set)
exportMethods(export)
exportMethods(import)
importClassesFrom(S4Vectors,Annotated)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,keytypes)
importFrom(AnnotationDbi,mapIds)
importFrom(BiocIO,export)
importFrom(BiocIO,import)
importFrom(BiocIO,resource)
importFrom(KEGGREST,keggGet)
importFrom(KEGGREST,keggList)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(dplyr,'%>%')
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,intersect)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(dplyr,tbl_nongroup_vars)
importFrom(dplyr,ungroup)
importFrom(dplyr,union)
importFrom(methods,coerce)
importFrom(methods,initialize)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setAs)
importFrom(methods,setOldClass)
importFrom(methods,slot)
importFrom(ontologyIndex,get_ontology)
importFrom(plyr,mapvalues)
importFrom(rlang,.data)
importFrom(rlang,enquo)
importFrom(rlang,quo_text)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tibble,enframe)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,unnest)
importFrom(utils,write.table)
