% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BiocSet-methods.R
\name{coerce}
\alias{coerce}
\alias{coerce,BiocSet,list-method}
\title{as("BiocSet", "list")}
\description{
as("BiocSet", "list")
}
